--------------------------------------------------------------------------------
-- LEON3 Usecase JRZ VECS - FH Technikum Wien (c) 2013/2014
-- File: assertion_checkers_p.vhd                                             --
-- Author: matthias                                                          --
-- Created on: 12.02.2014                                                     --
--                                                                            --
-- Package containing component declarations for assertion checkers           --
--------------------------------------------------------------------------------

library IEEE;
	use IEEE.std_logic_1164.all;
	use IEEE.std_logic_arith.all;
library work;


library grlib;
use grlib.amba.all;


package assertion_checkers_p is

type assertion_stat_t is record
  --pending :std_logic;
  --done :std_logic;
  failed :std_logic;
end record;

constant ASS_STAT_WIDTH : integer := 32;

type assertion_status_t is array(ASS_STAT_WIDTH-1 downto 0) of assertion_stat_t;


---administrative components
	component wait_counter is
	  generic(
	    G_LOWER_BOUND :integer;
	    G_UPPER_BOUND :integer
	  );
	  port (
		clk_i     : in std_logic;
		reset_i   : in std_logic;
		
		enable_i :in std_logic;
			
		--bounds reached
		lower_thr_o :out std_logic;
		upper_thr_o :out std_logic
	  );
	end component wait_counter;

	component assertion_collector is
	generic (
        pindex   : integer := 0;
        paddr    : integer := 0;
        pmask    : integer := 16#fff#;
        pirq     : integer := 0;
        nbits    : integer range 1 to 64 := 32;
        rstval   : integer := 0;
        rstval2  : integer := 0;
        extrst   : integer := 0
        );
	port (
        rst    : in  std_ulogic;
        clk    : in  std_ulogic;
        apbi   : in  apb_slv_in_type;
        apbo   : out apb_slv_out_type;
        fatal_fail_o :out std_logic_vector(ASS_STAT_WIDTH-1 downto 0);
        refeed_self_reset: in std_logic_vector (1 downto 0);
        self_reset_o :out std_logic;
        ass_stat : in assertion_status_t;
        resval : in  std_logic_vector(nbits-1 downto 0) := (others => '0')
        );
	end component assertion_collector;
--end administrative components	


--begin checkers

	component rule5_addr_stable_during_transition is
	port (
		clk_i     : in std_logic;
		reset_ni   : in std_logic;
		-- assertion inputs --
		psel_i : in std_logic_vector(0 to 15);
		penable_i :in std_ulogic;
		paddr_i :std_logic_vector(31 downto 0);
		pwrite_i :std_ulogic;
		pwdata_i :std_logic_vector(31 downto 0);

		-- assertion status --
		pending_o : out std_logic;
		done_o    : out std_logic;
		failed_o  : out std_logic

	);
	end component rule5_addr_stable_during_transition;

	component rule22_htrans_change_when_hready_low is
	port (
		clk_i     : in std_logic;
		reset_i   : in std_logic;
		-- assertion inputs --
		hready	: in std_logic;
		htrans : in std_logic_vector(1 downto 0);

		-- assertion status --
		pending_o : out std_logic;
		done_o    : out std_logic;
		failed_o  : out std_logic

	);
	end component rule22_htrans_change_when_hready_low;


	component uart_obeys_ctsn is
	port (
		clk_i     : in std_logic;
		reset_i   : in std_logic;
		-- assertion inputs --
		txd :in std_logic;
		ctsn :in std_logic;

		-- assertion status --
		pending_o : out std_logic;
		done_o    : out std_logic;
		failed_o  : out std_logic

	);
	end component uart_obeys_ctsn;

	
	component i2c_liveness is
	port (
		clk_i     : in std_logic;
		reset_i   : in std_logic;
		-- assertion inputs --
		scl :in std_logic;
		sda :in std_logic;
			
		-- assertion status --
		pending_o : out std_logic;
		done_o    : out std_logic;
		failed_o  : out std_logic
	
	);

	end component i2c_liveness;
	
	component i2c_always_start_after_stop is
	port (
		clk_i : in std_logic;
		reset_i : in std_logic;
		
		sda_i : in std_ulogic;
		scl_i : in std_ulogic;
		
		-- assertion status --
		pending_o : out std_logic;
		done_o    : out std_logic;
		failed_o  : out std_logic
	);
	end component i2c_always_start_after_stop;
	
	
	component timer_interrupt_cpu1 is
	port (
		clk_i     : in std_logic;
		reset_i   : in std_logic;
		-- assertion inputs --
		haddr_0	: in std_logic_vector(31 downto 0);
		hwrite_0	: in std_logic;
		hwdata_0	: in std_logic;
		
		haddr_1	: in std_logic_vector(31 downto 0);
		hwrite_1	: in std_logic;
		hwdata_1	: in std_logic;
		
		pirq	: in std_logic;
			
		-- assertion status --
		pending_o : out std_logic;
		done_o    : out std_logic;
		failed_o  : out std_logic
	
	);
	end component timer_interrupt_cpu1;
	
	
	component timer_interrupt_never_cpu0 is
		port(
			clk_i     : in std_logic;
			reset_i   : in std_logic;
			-- assertion inputs --
			haddr	: in std_logic_vector(31 downto 0);
			hwrite	: in std_logic;
			hwdata	: in std_logic;
			pirq	: in std_logic;
			
			-- assertion status --
			pending_o : out std_logic;
			done_o    : out std_logic;
			failed_o  : out std_logic
			
		);
	end component timer_interrupt_never_cpu0;

	component timer_interrupt_successive_cpu1 is
	port (
		clk_i     : in std_logic;
		reset_i   : in std_logic;
		-- assertion inputs --
		haddr_0	: in std_logic_vector(31 downto 0);
		hwrite_0	: in std_logic;
		hwdata_0	: in std_logic;
		
		pirq	: in std_logic;
			
		-- assertion status --
		pending_o : out std_logic;
		done_o    : out std_logic;
		failed_o  : out std_logic
	
	);
	end component timer_interrupt_successive_cpu1;
--end checkers

end package assertion_checkers_p;