--------------------------------------------------------------------------------
-- LEON3 Usecase JRZ VECS - FH Technikum Wien (c) 2013/2014
-- File: timer_interrupt_cpu1.vhd                                                      --
-- Author: christain (for 8051) matthias(for leon3)                                                         --
-- Created on: 26.2.2014                                                    --
--                                                                            --
-- Assertion which checks if the 
--
-- property start_after_stop is always (((rose(rtc_i2c_sda_i) and rtc_i2c_scl_i = 'H')) 
-- 	-> next ((rtc_i2c_sda_i = 'H' and rtc_i2c_scl_i = 'H')
-- until (fell(rtc_i2c_sda_i) and rtc_i2c_scl_i = 'H')));
--------------------------------------------------------------------------------



library IEEE;
	use IEEE.std_logic_1164.all;
	
entity i2c_always_start_after_stop is
	port (
		clk_i : in std_logic;
		reset_i : in std_logic;
		
		sda_i : in std_ulogic;
		scl_i : in std_ulogic;
		
		-- assertion status --
		pending_o : out std_logic;
		done_o    : out std_logic;
		failed_o  : out std_logic
		
	);
end entity i2c_always_start_after_stop;

architecture manual_rtl of i2c_always_start_after_stop is

	type t_i2c_state is (SAMPLE0, SAMPLE1, WAIT_FOR_STOP_CONDITION,WAIT_FOR_START_CONDITION);

	signal s_i2c_state : t_i2c_state;
	
	signal s_sda : std_ulogic_vector(1 downto 0);
	signal s_scl : std_ulogic_vector(1 downto 0);
	
	signal s_always_start_after_stop_failed_o : std_logic;
	signal s_pending : std_logic;
	signal s_done : std_logic;
	signal s_failed : std_logic;
	
begin

	p_check_always_start_after_stop : process (clk_i, reset_i) 
	begin
		if reset_i = '1' then
			-- async reset --
			s_i2c_state <= SAMPLE0;
			s_sda <= "00";
			s_scl <= "00";
			s_pending <= '0';
			s_done <= '0';
			s_failed <= '0';
			
			
		elsif rising_edge(clk) then
		
			s_always_start_after_stop_failed_o <= '0';
			
			s_sda(0) <= sda_i;
			s_sda(1) <= s_sda(0);
			
			s_scl(0) <= scl_i;
			s_scl(1) <= s_scl(0);
		
			case i2c_state is
				
				when SAMPLE0 =>
					-- sda(0) and scl(0) get valid after this cycle
					s_i2c_state <= SAMPLE1;
					
				when SAMPLE1 =>
					-- sda(1) and scl(1) get valid after this cycle
					s_i2c_state <= WAIT_FOR_STOP_CONDITION;
					
				when WAIT_FOR_STOP_CONDITION =>
					s_pending <= '1';
					if s_sda(1) = '0' and s_sda(0) = '1' and s_scl(1) = '1' and s_scl(0) = '1' then
						s_i2c_state <= WAIT_FOR_START_CONDITION;
					end if;
				
				when WAIT_FOR_START_CONDITION =>
				
					if s_sda(1) = '1' and s_sda(0) = '0' and s_scl(1) = '1' and s_scl(0) = '1' then
						-- detect start condition --
						s_i2c_state <= WAIT_FOR_STOP_CONDITION;
					elsif s_sda = '0' or s_scl = '0' then
						-- if change other than start condition, assertion failed --
						s_always_start_after_stop_failed_o <= '1';
					end if;	
		
			end case;
		
		end if;
	
	end process p_check_always_start_after_stop;

end manual_rtl