--------------------------------------------------------------------------------
-- File:rule22_htrans_change_when_hready_low.vhd                                                      --
-- Author: matthias                                                          --
-- Created on: 17.2.2014                                                      --
--                                                                            --
-- property rule22_htrans_change_when_hready_low(const master) is
--
-- always(prev(ahbmi.hready='0') ->
-- ((prev(ahbmo(master).htrans) = ahbmo(master).htrans) or
-- (prev(ahbmo(master).htrans) = AHB_HTRANS_IDLE and
-- ahbmo(master).htrans = AHB_HTRANS_NONSEQ)));
--------------------------------------------------------------------------------

library ieee;
	use ieee.std_logic_1164.all;
	use IEEE.std_logic_arith.all;

library grlib;
      use grlib.amba.all;
	
library work;
	use work.assertion_checkers_p.all;

entity rule22_htrans_change_when_hready_low is
	port (
		clk_i     : in std_logic;
		reset_i   : in std_logic;
		-- assertion inputs --
		hready	: in std_logic;
		htrans : in std_logic_vector(1 downto 0);

		-- assertion status --
		pending_o : out std_logic;
		done_o    : out std_logic;
		failed_o  : out std_logic

	);

end entity rule22_htrans_change_when_hready_low;


architecture rtl of rule22_htrans_change_when_hready_low is

  signal htrans_r, htrans_2_r	: std_logic_vector(1 downto 0);
  signal hready_r, hready_2_r	: std_logic;


  type state_t is (WAIT_S,WAIT_IRQ_ACK_S);
  signal state_act, state_nxt : state_t;


  signal pending, pending_r :std_logic;
  signal failed, failed_r :std_logic;
  signal done, done_r :std_logic;

begin

  p_check : process(clk_i,reset_i)
  begin
    if(reset_i = '0') then
      state_act <= WAIT_S;

      htrans_r <= (others => '0');
      htrans_2_r <= (others => '0');
      hready_r <= '0';
      hready_2_r <= '0';

      done_r <= '0';
      pending_r <= '0';
      failed_r <= '0';

    elsif rising_edge(clk_i) then

      state_act <= state_nxt;

      htrans_r <= htrans;
      htrans_2_r <= htrans_r;

      hready_r <= hready;
      hready_2_r <= hready_r;

      done_r <= done;
      pending_r <= pending;
      failed_r <= failed;

    end if;


  end process;


  p_check_comb: process(done_r, pending_r, failed_r, htrans_r, htrans_2_r, hready_r, hready_2_r, state_act) is
  begin
    state_nxt <= state_act;
    done <= done_r;
    pending <= pending_r;
    failed <= failed_r;

    case state_act is
      when WAIT_S =>
	pending <= '0';
	done <= '0';
	failed <= '0';

	if(hready_2_r = '0') then
	  if( (htrans_2_r = htrans_r) or
	    ( (htrans_2_r = HTRANS_IDLE) and
	      (htrans_r = HTRANS_NONSEQ))) then
	    done <= '1';
	  else
	    failed <= '1';
	  end if;
	end if;

      when others => null;
    end case;

  end process;

  done_o <= done or done_r;
  pending_o <= pending or pending_r;
  failed_o <= failed or failed_r;


end architecture rtl;
