--------------------------------------------------------------------------------
-- LEON3 Usecase JRZ VECS - FH Technikum Wien (c) 2013/2014
-- File:rule5_addr_stable_during_transition.vhd                                            --
-- Author: matthias                                                          --
-- Created on: 2.4.2014                                                      --
--                                                                            --
--
--property  rule5_addr_stable_during_transition is
--       always(((prev(apbi.psel) /= "0000000000000000" and prev(apbi.penable) = '0') and
--                    (apbi.psel   /= "0000000000000000" and      apbi.penable  = '1')) -> (
--                                  apbi.paddr =  prev(apbi.paddr)   and
--                                  apbi.pwrite = prev(apbi.pwrite) and
--                                  apbi.pwdata = prev(apbi.pwdata) and
 --                                 apbi.psel   = prev(apbi.psel)));
--------------------------------------------------------------------------------

library ieee;
	use ieee.std_logic_1164.all;
	use IEEE.std_logic_arith.all;

library grlib;
      use grlib.amba.all;
	
library work;
	use work.assertion_checkers_p.all;

entity rule5_addr_stable_during_transition is
	port (
		clk_i     : in std_logic;
		reset_ni   : in std_logic;
		-- assertion inputs --
		psel_i : in std_logic_vector(0 to 15);
		penable_i :in std_ulogic;
		paddr_i :std_logic_vector(31 downto 0);
		pwrite_i :std_ulogic;
		pwdata_i :std_logic_vector(31 downto 0);

		-- assertion status --
		pending_o : out std_logic;
		done_o    : out std_logic;
		failed_o  : out std_logic

	);

end entity rule5_addr_stable_during_transition;


architecture rtl of rule5_addr_stable_during_transition is

		signal psel, psel_r,psel_2r :std_logic_vector(0 to 15);
		signal penable, penable_r, penable_2r: std_ulogic;
		signal paddr, paddr_r, paddr_2r :std_logic_vector(31 downto 0);
		signal pwrite, pwrite_r, pwrite_2r :std_ulogic;
		signal pwdata, pwdata_r, pwdata_2r :std_logic_vector(31 downto 0);


  type state_t is (WAIT_S);
  signal state_act, state_nxt : state_t;


  signal pending, pending_r :std_logic;
  signal failed, failed_r :std_logic;
  signal done, done_r :std_logic;

begin

  p_check : process(clk_i,reset_ni)
  begin
    if(reset_ni = '0') then
      state_act <= WAIT_S;

      psel_r <= (others => '0');
      penable_r <= '0';
      paddr_r <= (others => '0');
      pwrite_r <= '0';
      pwdata_r <= (others => '0');

			psel_2r <= (others => '0');
      penable_2r <= '0';
      paddr_2r <= (others => '0');
      pwrite_2r <= '0';
      pwdata_2r <= (others => '0');
      
      done_r <= '0';
      pending_r <= '0';
      failed_r <= '0';

    elsif rising_edge(clk_i) then

      state_act <= state_nxt;

			psel_r <= psel_i;
			psel_2r <= psel_r;

			penable_r <= penable_i;
			penable_2r <= penable_r;

			paddr_r <= paddr_i;
			paddr_2r <= paddr_r;

			pwdata_r <= pwdata_i;
			pwdata_2r <= pwdata_r;

      pwrite_r <= pwrite_i;
      pwrite_2r <= pwrite_r;

      done_r <= done;
      pending_r <= pending;
      failed_r <= failed;

    end if;


  end process;


  p_check_comb: process(done_r, pending_r, failed_r, psel_r, psel_2r, penable_r,
  penable_2r, paddr_r, paddr_2r, pwdata_r, pwdata_2r, state_act, pwrite_2r, pwrite_r) is
  begin
    state_nxt <= state_act;
    done <= done_r;
    pending <= pending_r;
    failed <= failed_r;

    case state_act is
      when WAIT_S =>
				pending <= '0';
				done <= '0';
				failed <= '0';

				if psel_2r /= "0000000000000000" and penable_2r = '0' and
					psel_r /= "0000000000000000" and penable_r = '1' then
					
					if paddr_r =  paddr_2r and pwrite_r = pwrite_2r and
						pwdata_r = pwdata_2r and psel_r = psel_2r then
						done <= '1';
					else
						failed <= '1';
					end if;
					
				end if;

      when others => null;
    end case;

  end process;

  done_o <= done or done_r;
  pending_o <= pending or pending_r;
  failed_o <= failed or failed_r;


end architecture rtl;
