--------------------------------------------------------------------------------
-- LEON3 Usecase JRZ VECS - FH Technikum Wien (c) 2013/2014
-- File: timer_interrupt_never_cpu0.vhd                                                      --
-- Author: matthias                                                          --
-- Created on: 12.2.2014                                                      --
--                                                                            --
-- Assertion which checks if the GPTIMER tinterrupt is never assigned to cpu0     
-- covers timer initialization--
--property timer_interrupt_never_cpu0 is never ((ahbmo(0).haddr = X"80000240" and ahbmo(0).hwrite = '1' and ahbmo(0).hwdata(8) = '1'));
--------------------------------------------------------------------------------

library ieee;
	use ieee.std_logic_1164.all;
	use IEEE.std_logic_arith.all;
library work;
	use work.assertion_checkers_p.all;
	
entity timer_interrupt_never_cpu0 is
	port (
		clk_i     : in std_logic;
		reset_i   : in std_logic;
		-- assertion inputs --
		haddr	: in std_logic_vector(31 downto 0);
		hwrite	: in std_logic;
		hwdata	: in std_logic;
		pirq	: in std_logic;
			
		-- assertion status --
		pending_o : out std_logic;
		done_o    : out std_logic;
		failed_o  : out std_logic
	
	);

end entity timer_interrupt_never_cpu0;


architecture rtl of timer_interrupt_never_cpu0 is

  signal haddr_r	: std_logic_vector(31 downto 0);
  signal hwrite_r	: std_logic;
  signal hwdata_r	: std_logic;
  signal pirq_r 	: std_logic;

begin

  p_check : process(clk_i,reset_i)
  begin
    if(reset_i = '0') then

      haddr_r <= (others => '0');
      hwrite_r <= '0';
      hwdata_r <= '0';
      pirq_r <= '0';
      
      done_o <= '0';
      pending_o <= '0';
      failed_o <= '0';
    
    elsif rising_edge(clk_i) then
      
      haddr_r <= haddr;
      hwrite_r <= hwrite;
      hwdata_r <= hwdata;
      pirq_r <= pirq;
      
      
      done_o <= '0';
      pending_o <= '0';
      failed_o <= '0';
      
    
      if(haddr_r = X"80000240" and hwrite_r = '1' and hwdata_r = '1') then
	done_o <= '1';
	failed_o <= '1';
      end if;
    
    
    end if;
  
  
  end process;



end architecture rtl;
