--------------------------------------------------------------------------------
-- LEON3 Usecase JRZ VECS - FH Technikum Wien (c) 2013/2014
-- File:timer_interrupt_successive_cpu1.vhd                                                      --
-- Author: matthias                                                          --
-- Created on: 17.2.2014                                                      --
--                                                                            --
-- Assertion which checks if the GPTIMER interrupt is always serviced by cpu1   
-- covers interrupt acknowledge
--property subsequent_interrupts is always((apbo(3).pirq(8) = '1') -> {[*1 to 218];ahbmo(1).haddr = X"80000318" and ahbmo(1).hwrite = '1' and ahbmo(1).hwdata(4) = '1'});
--------------------------------------------------------------------------------

library ieee;
	use ieee.std_logic_1164.all;
	use IEEE.std_logic_arith.all;
library work;
	use work.assertion_checkers_p.all;
	
entity timer_interrupt_successive_cpu1 is
	port (
		clk_i     : in std_logic;
		reset_i   : in std_logic;
		-- assertion inputs --
		haddr_0	: in std_logic_vector(31 downto 0);
		hwrite_0	: in std_logic;
		hwdata_0	: in std_logic;
		
		pirq	: in std_logic;
			
		-- assertion status --
		pending_o : out std_logic;
		done_o    : out std_logic;
		failed_o  : out std_logic
	
	);

end entity timer_interrupt_successive_cpu1;


architecture rtl of timer_interrupt_successive_cpu1 is

  signal haddr_0_r	: std_logic_vector(31 downto 0);
  signal hwrite_0_r	: std_logic;
  signal hwdata_0_r	: std_logic;
 
  
  
  signal pirq_r	: std_logic;

  type state_t is (WAIT_S,WAIT_IRQ_ACK_S);
  signal state_act, state_nxt : state_t;
  
  
  signal pending, pending_r :std_logic;
  signal failed, failed_r :std_logic;
  signal done, done_r :std_logic;
  
  --wait counter
  signal my_reset_n :std_logic;
  signal cnt_enable :std_logic;
  signal lower_thr :std_logic;
  signal upper_thr :std_logic;
  
begin

  wait_counter_0 : wait_counter
	generic map(
	  G_LOWER_BOUND  =>0,--0 because 1 cycle is already consumed by state change from WAIT_S to WAIT_FOR_*_S
	  G_UPPER_BOUND => 230-- -1 because 1 cycle is already consumed by state change from WAIT_S to WAIT_FOR_*_S
	)
	port map(
		clk_i  =>  clk_i,
		reset_i  =>  my_reset_n,	
		enable_i => cnt_enable,
		lower_thr_o => lower_thr,
		upper_thr_o => upper_thr	
	);


  p_check : process(clk_i,reset_i)
  begin
    if(reset_i = '0') then
      state_act <= WAIT_S;
       
      haddr_0_r <= (others => '0');
      hwrite_0_r <= '0';
      hwdata_0_r <= '0';
      pirq_r <= '0';
      
      done_r <= '0';
      pending_r <= '0';
      failed_r <= '0';
    
    elsif rising_edge(clk_i) then
      
      state_act <= state_nxt;
      

      haddr_0_r <= haddr_0;
      hwrite_0_r <= hwrite_0;
      hwdata_0_r <= hwdata_0;
      pirq_r <= pirq;
      
      done_r <= done;
      pending_r <= pending;
      failed_r <= failed;
    
    end if;
  
  
  end process;
  
  
  p_check_comb: process(done_r, pending_r, failed_r, lower_thr,upper_thr,haddr_0_r,hwrite_0_r,hwdata_0_r,pirq_r,state_act) is
  begin
    state_nxt <= state_act;
    done <= done_r;
    pending <= pending_r;
    failed <= failed_r;
    my_reset_n <= '1';
    cnt_enable <= '0';
    
    case state_act is
      when WAIT_S =>
	pending <= '0';
	done <= '0';
	failed <= '0';
	
	if(pirq_r = '1') then
	  pending <= '1';
	  my_reset_n <= '0';--ready counter
	  state_nxt <= WAIT_IRQ_ACK_S;
	end if;
  
      when WAIT_IRQ_ACK_S =>
	cnt_enable <= '1';
	if (upper_thr = '0' and lower_thr = '1') and (haddr_0_r = X"80000318" and hwrite_0_r = '1' and hwdata_0_r = '1') then
	  done <= '1';
	  pending <= '0';
	  cnt_enable <= '0';
	  state_nxt <= WAIT_S;
	elsif (upper_thr = '1' and lower_thr = '1') then
	  done <= '1';
	  failed <= '1';
	  pending <= '0';
	  cnt_enable <= '0';
	  state_nxt <= WAIT_S;
	end if;
    
      when others => null;
    end case;
  
  end process;

  done_o <= done or done_r;
  pending_o <= pending or pending_r;
  failed_o <= failed or failed_r;
  

end architecture rtl;
