--------------------------------------------------------------------------------
-- LEON3 Usecase JRZ VECS - FH Technikum Wien (c) 2013/2014
-- File:uart_obeys_ctsn.vhd                                                      --
-- Author: matthias                                                          --
-- Created on: 17.2.2014                                                      --
--                                                                            --
--
--uart transmission obeys ctsn signal, transmission may take 10 to 11 bits (without/with parity)
--  property uart_obeys_ctsn is always(const frame_length) ({u1o.txd = '0' and prev(u1o.txd) = '1'} |=>
--				     {u1i.ctsn = '0'}[*frame_length]
--  );
--------------------------------------------------------------------------------

library ieee;
	use ieee.std_logic_1164.all;
	use IEEE.std_logic_arith.all;
library work;
	use work.assertion_checkers_p.all;

entity uart_obeys_ctsn is
	port (
		clk_i     : in std_logic;
		reset_i   : in std_logic;
		-- assertion inputs --
		txd :in std_logic;
		ctsn :in std_logic;

		-- assertion status --
		pending_o : out std_logic;
		done_o    : out std_logic;
		failed_o  : out std_logic

	);

end entity uart_obeys_ctsn;


architecture rtl of uart_obeys_ctsn is

  
  signal txd_r, txd_prev_r	: std_logic;
  signal ctsn_r	: std_logic;


  type state_t is (WAIT_S,CTSN_OBEY_S);
  signal state_act, state_nxt : state_t;


  signal pending, pending_r :std_logic;
  signal failed, failed_r :std_logic;
  signal done, done_r :std_logic;

  --wait counter
  signal wait_reset_n :std_logic;
  signal cnt_enable :std_logic;
  signal lower_thr :std_logic;
  signal upper_thr :std_logic;

begin

  wait_counter_0 : wait_counter
	generic map(
	  G_LOWER_BOUND  =>4398, --from 0 on this are 4399 steps
	  G_UPPER_BOUND => 4398)
	port map(
		clk_i  =>  clk_i,
		reset_i  =>  wait_reset_n,
		enable_i => cnt_enable,
		lower_thr_o => lower_thr,
		upper_thr_o => upper_thr
	);


  p_check : process(clk_i,reset_i)
  begin
    if(reset_i = '0') then
      state_act <= WAIT_S;

      txd_r <= '1';
      txd_prev_r <= '1';
      ctsn_r <=  '1';

      done_r <= '0';
      pending_r <= '0';
      failed_r <= '0';

    elsif rising_edge(clk_i) then

      state_act <= state_nxt;


      txd_r <= txd;
      txd_prev_r <= txd_r;
      ctsn_r <= ctsn;

      done_r <= done;
      pending_r <= pending;
      failed_r <= failed;

    end if;


  end process;


  p_check_comb: process(done_r, pending_r, failed_r, lower_thr,upper_thr,txd_prev_r, txd_r, ctsn_r,state_act) is
  begin
    state_nxt <= state_act;
    done <= done_r;
    pending <= pending_r;
    failed <= failed_r;
    wait_reset_n <= '1';
    cnt_enable <= '0';

    case state_act is
      when WAIT_S =>
				pending <= '0';
				done <= '0';
				failed <= '0';

			if(txd_r = '0' and txd_prev_r = '1') then
				pending <= '1';
				wait_reset_n <= '0';--ready counter
	  
				state_nxt <= CTSN_OBEY_S;
				if(ctsn_r = '1') then
					pending <= '0';
					failed <= '1';
					state_nxt <= WAIT_S;
				end if;
			end if;


		when CTSN_OBEY_S =>
			cnt_enable <= '1';
			if(ctsn_r = '1') then
				done <= '1';
				failed <= '1';
				pending <= '0';
				cnt_enable <= '0';
				state_nxt <= WAIT_S;
			elsif (lower_thr = '1' and ctsn_r = '0')then
				done <= '1';
				pending <= '0';
				cnt_enable <= '0';
				state_nxt <= WAIT_S;
			end if;

		when others => null;
	end case;

  end process;

  done_o <= done or done_r;
  pending_o <= pending or pending_r;
  failed_o <= failed or failed_r;


end architecture rtl;
