--------------------------------------------------------------------------------
-- LEON3 Usecase JRZ VECS - FH Technikum Wien (c) 2013/2014
-- File: wait_counter.vhd                                                      --
-- Author: matthias                                                          --
-- Created on: 12.2.2014                                                      --
--                                                                            --
-- a wait counter, instantiate if simple wait states would not cut it                                                         --
--------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.math_real.log2;
use ieee.math_real.ceil;

library work;
use work.assertion_checkers_p.all;


entity wait_counter is
  generic(
    G_LOWER_BOUND : integer;
    G_UPPER_BOUND : integer

    );
  port (
    clk_i   : in std_logic;
    reset_i : in std_logic;

    enable_i : in std_logic;

    --bounds reached
    lower_thr_o : out std_logic;
    upper_thr_o : out std_logic

    );
end entity wait_counter;


architecture rtl of wait_counter is

  constant UPPER_BOUND_WIDTH : integer := integer(ceil(log2(real(G_UPPER_BOUND-1))));

  signal count, count_r                 : unsigned(UPPER_BOUND_WIDTH-1 downto 0);
  signal lower, lower_r, upper, upper_r : std_logic;

  constant LOWER_BOUND : unsigned(UPPER_BOUND_WIDTH-1 downto 0) := conv_unsigned(G_LOWER_BOUND, UPPER_BOUND_WIDTH);
  constant UPPER_BOUND : unsigned(UPPER_BOUND_WIDTH-1 downto 0) := conv_unsigned(G_UPPER_BOUND, UPPER_BOUND_WIDTH);
  
begin

  p_count : process(clk_i, reset_i)
  begin
    if(reset_i = '0') then
      count_r <= (others => '0');
      lower_r <= '0';
      upper_r <= '0';
      
    elsif rising_edge(clk_i) then
      count_r <= count;
      lower_r <= lower;
      upper_r <= upper;
    end if;
    
    
  end process;


  p_count_comb : process(lower_r, upper_r, count_r, enable_i) is
  begin
    
    count <= count_r;
    lower <= lower_r;
    upper <= upper_r;


    if enable_i = '1' then
      
      if(count_r = LOWER_BOUND) then
        lower <= '1';
      elsif(count_r = UPPER_BOUND) then
        upper <= '1';
      end if;

      count <= count_r + '1';
    end if;

    lower_thr_o <= lower_r;
    upper_thr_o <= upper_r;
    
  end process;
  

end architecture rtl;
