--------------------------------------------------------------------------------
-- LEON3 Usecase JRZ VECS - FH Technikum Wien (c) 2013/2014
-- File: fault_injection_decoder.vhd                                      --
-- Author: matthias                                                         --
-- Created on: 25.3.2014                                                     --
--                                                                            --
-- implements a onehot0 encoded modulo G_ADDRESS_WIDTH fault injection decoder
--
--------------------------------------------------------------------------------

library ieee;
    use ieee.std_logic_1164.all;
    use ieee.std_logic_arith.all;

library work;
    use work.fault_injection_p.all;

entity fault_injection_decoder is
    generic (
        G_ADDRESS_WIDTH :integer;
        G_INVERT_BCD_ADDRESS :boolean := false;
        G_INVERT_ONEHOT0_ADDRESS :boolean := false
    );
    port (
        clk_i     : in std_logic;
        reset_n_i : in std_logic;
        next_address_i : in std_logic;
				computed_address_o :out std_logic_vector(G_ADDRESS_WIDTH-1 downto 0);
				computed_address_onehot0_o :out std_logic_vector((2**G_ADDRESS_WIDTH)-1 downto 0)
    );
end entity;

architecture rtl of fault_injection_decoder is

	constant ADDRESS_ONHEOT_WIDTH :integer := (2**G_ADDRESS_WIDTH);--because of zero
	constant ZERO :integer := 0;
	
	signal next_address_r :std_logic_vector(1 downto 0);
	signal computed_address, computed_address_r : unsigned(G_ADDRESS_WIDTH-1 downto 0);
	signal computed_address_onehot0, computed_address_onehot0_r : std_logic_vector(ADDRESS_ONHEOT_WIDTH-1 downto 0);
	


begin


    p_sync_decoder: process(clk_i, reset_n_i)
    begin
        if reset_n_i = '0' then
					computed_address_r <= (others => '0');
          computed_address_onehot0_r <= (others => '0');
          next_address_r <= (others => '0');
        elsif rising_edge(clk_i) then
					next_address_r(0) <= next_address_i;
					next_address_r(1) <= next_address_r(0);
					computed_address_onehot0_r <= computed_address_onehot0;
					computed_address_r <= unsigned(computed_address);
        end if;
    end process p_sync_decoder;

    p_comb_decoder :process(computed_address_r,computed_address_onehot0_r,next_address_r)
    begin
			computed_address <= computed_address_r;
			computed_address_onehot0 <= computed_address_onehot0_r;

			--trigger on edge event
			if (next_address_r(1) = not C_BUTTON_PRESSED) and (next_address_r(0) = C_BUTTON_PRESSED) then
				computed_address <= computed_address_r + '1';
				
				if computed_address_onehot0_r = std_logic_vector(conv_unsigned(ZERO,ADDRESS_ONHEOT_WIDTH)) then
					computed_address_onehot0 <= computed_address_onehot0_r(ADDRESS_ONHEOT_WIDTH-2 downto 0) & '1';
				elsif computed_address_onehot0_r((computed_address_onehot0_r'left)-1) = '1' then
					computed_address_onehot0 <= (others => '0');
				else
					computed_address_onehot0 <= computed_address_onehot0_r(ADDRESS_ONHEOT_WIDTH-2 downto 0) & '0';
				end if;
			end if;

			if G_INVERT_BCD_ADDRESS then
				for i in 0 to G_ADDRESS_WIDTH-1 loop
					computed_address_o(i) <= std_logic(not computed_address_r(i));
				end loop;
			else
				computed_address_o <= std_logic_vector(computed_address_r);
			end if;

			if G_INVERT_ONEHOT0_ADDRESS then
				for i in 0 to ADDRESS_ONHEOT_WIDTH-1 loop
					computed_address_onehot0_o(i) <= computed_address_onehot0_r(i);
				end loop;
			else
				computed_address_onehot0_o <= computed_address_onehot0_r;
			end if;
			
    end process p_comb_decoder;



end rtl;