--------------------------------------------------------------------------------
-- LEON3 Usecase JRZ VECS - FH Technikum Wien (c) 2013/2014
-- File: fault_injection_p.vhd                                                --
-- Author: christian, matthias                                                          --
-- Created on: 14.1.2014
-- added fault injection decoder 25.3.2014
--                                                                            --
-- Component declarations, constants and functions for fault injection unit   --
-- and button synchronizer                                                    --
--------------------------------------------------------------------------------

library ieee;
    use ieee.std_logic_1164.all;
    use ieee.std_logic_arith.all;

package fault_injection_p is
    
    type t_operation is (OP_AND,OP_OR,OP_XOR,OP_OVW);

    constant C_BUTTON_PRESSED : std_logic := '0';

		component fault_injection_decoder is
    generic (
        G_ADDRESS_WIDTH :integer;
        G_INVERT_BCD_ADDRESS :boolean := false;
        G_INVERT_ONEHOT0_ADDRESS :boolean := false
    );
    port (
        clk_i     : in std_logic;
        reset_n_i : in std_logic;
        next_address_i : in std_logic;
				computed_address_o :out std_logic_vector(G_ADDRESS_WIDTH-1 downto 0);
				computed_address_onehot0_o :out std_logic_vector((2**G_ADDRESS_WIDTH)-1 downto 0)
    );
		end component;

    
    component fault_injection_unit is
    generic(
        G_ASYNC_TRIGGER          : boolean;
		G_DEBOUNCE_COUNTER_WIDTH : natural range 1 to natural'right := 1;
        G_OPERATION              : t_operation := OP_OVW;
        G_INJECT_SINGLE_SHOT     : boolean := true;
        G_RANDOMIZE              : boolean := false;
        G_DATA_WIDTH             : natural := 8;
        G_FI_DATA                : integer := 16#FF#;
        G_LFSR_POLY              : integer := 16#71#;
		G_LFSR_WIDTH             : integer := 8
    );
    port(
        clk_i                : in  std_logic;
        reset_n_i            : in  std_logic;
        trigger_i            : in  std_logic;
    
        data_i              : in  std_logic_vector(G_DATA_WIDTH-1 downto 0);
        data_o              : out std_logic_vector(G_DATA_WIDTH-1 downto 0)
    );
    end component;
    
    component fault_injection_button_sync is
    generic (
        G_DEBOUNCE_COUNTER_WIDTH : natural;
        G_DEBOUNCE_SIGNAL_WIDTH  : natural;
        G_SINGLE_SHOT_MASK       : natural
    );
    port (
        clk_i     : in std_logic;
        reset_n_i : in std_logic;
        async_i   : in std_logic_vector(G_DEBOUNCE_SIGNAL_WIDTH-1 downto 0);
        sync_o    : out std_logic_vector(G_DEBOUNCE_SIGNAL_WIDTH-1 downto 0)
    );
    end component;

    function nat2bool(a : natural) return boolean;
    function bool2nat(a : boolean) return natural;
	function numbits(a : natural) return natural;

    
end package;

package body fault_injection_p is
    
	--
	-- Converts a natural number to a boolean (0: false, >0: true)
	--
    function nat2bool(a : natural) return boolean is
    begin
        if a = 0 then
            return false;
        else
            return true;
        end if;
    end function nat2bool;
    
	--
	-- Converts a boolean to a natural number (false: 0, true: 1)
	--
    function bool2nat(a : boolean) return natural is
    begin
        if a = false then
            return 0;
        else
            return 1;
        end if;
    end function bool2nat;

	--
	-- Returns the number of bits needed to represent an integer
	--
	function numbits(a : natural) return natural is
	begin
		for i in 0 to 31 loop
			if 2**i > a then 
				return i;
			end if;
		end loop;
	end function numbits;

end package body;
