

-- property first_interrupt is always ((ahbmo(0).haddr = X"80000240" and ahbmo(0).hwrite = '1' and ahbmo(0).hwdata(8) = '1') -> (
--            {apbo(3).pirq(8) = '0'[*];apbo(3).pirq(8) = '1'}|=>{apbo(3).pirq(8) = '0'[*];(ahbmo(0).haddr = X"80000318" and ahbmo(0).hwrite = '1' and ahbmo(0).hwdata(4) = '1')}));
--            
library ieee;
	use ieee.std_logic_1164.all;
	use ieee.std_logic_arith.all;
	
entity first_interrupt_checker is
	port(
		clk_i     : in std_logic;
		reset_i   : in std_logic;
		
		haddr_i    : in std_logic_vector(31 downto 0);
		hwrite_i   : in std_logic;
        hwdata_8_i : in std_logic;
        hwdata_4_i : in std_logic;
        pirq_i     : in std_logic;
		
        pending_o  : out std_logic;
        done_o     : out std_logic;
		failed_o   : out std_logic
	);

end entity first_interrupt_checker;

architecture rtl of first_interrupt_checker is

    type t_state is (WAIT_CONFIGURATION, WAIT_INTERRUPT, WAIT_ACKNOWLEDGE);

    signal s_state    : t_state;
    signal s_haddr    : std_logic_vector(31 downto 0);
    signal s_hwrite   : std_logic;
    signal s_hwdata_8 : std_logic;
    signal s_hwdata_4 : std_logic;
    signal s_pirq     : std_logic;

begin

    p_seq : process(clk_i, reset_i)
    begin
        if reset_i = '1' then
            s_haddr    <= (others => '0');
            s_hwrite   <= '0';
            s_hwdata_8 <= '0';
            s_hwdata_4 <= '0';
            s_pirq     <= '0';

            pending_o  <= '0';
            done_o     <= '0';
            failed_o   <= '0';

        elsif rising_edge(clk_i) then
            
            failed_o   <= '0';

            case s_state is
                when WAIT_CONFIGURATION =>
                    
                    pending_o  <= '0';
                    done_o     <= '0';

                    if s_haddr = X"80000240" and s_hwrite = '1' and s_hwdata_8 = '1' then
                        s_state <= WAIT_INTERRUPT;
                        pending_o <= '1';
                    end if;
                when WAIT_INTERRUPT =>
                    if s_pirq = '1' then
                        s_state <= WAIT_ACKNOWLEDGE;
                    end if;
                when WAIT_ACKNOWLEDGE =>
                    if s_haddr = X"80000318" and s_hwrite = '1' and s_hwdata_4 = '1' then
                        done_o     <= '1';
                        s_state <= WAIT_CONFIGURATION;
                    end if;
                when others =>
                    null;
            end case;

        end if;
    end process p_seq;


end rtl;

