--------------------------------------------------------------------------------
-- LEON3 Usecase JRZ VECS - FH Technikum Wien (c) 2013/2014
-- PRELIMINARY TOPLEVEL DESIGN FILE                                           --
-- Created on: 07.11.2013                                                     --
-- Author: christian    , matthias                                                      --
--------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
library work;
use work.config.all;
use work.fault_injection_p.all;

entity leon3mp_top is
  port(
	resetn	    : in  std_logic; --key[0]
	clock_50	: in  std_logic;
	
	-- BEGIN added for fault injection --
		
	user_pb_i   : in std_logic_vector(1 downto 0);                          -- Pressed = '0'
		
        -- END added for fault injection --
	
	-- BEGIN added for assertion collection enable --
		
	user_dipsw_i : in std_logic_vector(3 downto 0);                         -- To board edge = '1'
		
	-- END added for assertion collection enable --
		
	-- BEGIN added for i2c --
	i2c_scl_io : inout std_logic;                                           -- Dev Board I2C  (RTC, LCD, EEPROM, POWER MGMT, ...)
	i2c_sda_io : inout std_logic;                                           -- Dev Board I2C  (RTC, LCD, EEPROM, POWER MGMT, ...)
	-- END added for i2c --
	

	ledg_o        : out std_logic_vector(3 downto 0);                         -- 4 FPGA LEDs
		
	hsmc_ledg_o_p : out std_logic_vector(9 downto 0);                       -- 10 green LEDs on the HSMC "Debug Header" expansion board (anode)      (LTST-C195GEKT Pin 2)
	hsmc_ledg_o_n : out std_logic_vector(9 downto 0);                       -- 10 green LEDs on the HSMC "Debug Header" expansion board (cathode)    (LTST-C195GEKT Pin 4)
	hsmc_ledr_o_p : out std_logic_vector(9 downto 0);                       -- 10 red LEDs on the HSMC "Debug Header" expansion board (anode)        (LTST-C195GEKT Pin 1)
	hsmc_ledr_o_n : out std_logic_vector(9 downto 0)                        -- 10 red LEDs on the HSMC "Debug Header" expansion board (cathode)      (LTST-C195GEKT Pin 3)
	
  );
end;
	
architecture rtl of leon3mp_top is

component leon3mp
  generic (
    fabtech   : integer := CFG_FABTECH;
    memtech   : integer := CFG_MEMTECH;
    padtech   : integer := CFG_PADTECH;
    clktech   : integer := CFG_CLKTECH;
    disas     : integer := CFG_DISAS;	-- Enable disassembly to console
    dbguart   : integer := CFG_DUART;	-- Print UART on console
    pclow     : integer := CFG_PCLOW;
    SIMULATION :integer := 0
  );
  port (
    resetn	: in  std_logic; --key[0]
    clock_50	: in  std_logic;
    err_o       : out std_logic_vector(3 downto 0);
    errorn	: out std_logic; --ledr[0], error from LEON3 DSU
    uart_txd  	: out std_logic;		-- DSU, uart tx data
    uart_rxd  	: in  std_logic;		-- DSU,uart rx data
    dsubre  	: in std_logic;  --key[1], used to put processor in debug mode.
    dsuact  	: out std_logic; --ledr[1]
    i2c_scl	: inout std_logic;
    i2c_sda	: inout std_logic;
    gpio_0_final      : out std_logic_vector(CFG_GRGPIO_WIDTH-1 downto 0); 	-- I/O port 0
    gpio_1      : out std_logic_vector(CFG_GRGPIO2_WIDTH-1 downto 0); 	-- I/O port 1  
    fault_injection_buttons_async : in std_logic_vector(1 downto 0);
    ledg : out std_logic_vector(3 downto 0);
    dipsw_n : in std_logic_vector(3 downto 0);
    sw      	: in std_logic_vector(0 to 2) := "000" -- switch for jumper configuratioin, connects debug uart when sw(0) = '0'

    );
end component;

	signal s_gpio_0 : std_logic_vector(CFG_GRGPIO_WIDTH-1 downto 0);
	signal s_gpio_1 : std_logic_vector(CFG_GRGPIO2_WIDTH-1 downto 0);
	
	signal s_err :std_logic_vector(3 downto 0);

begin



hsmc_ledg_o_p <= s_gpio_0(9 downto 0); 
--hsmc_ledr_o_p <= s_gpio_0(19 downto 10);
hsmc_ledr_o_p <= s_gpio_1(9 downto 0);--red leds on gpio 1


top: entity work.leon3mp
  generic map(
    fabtech   => CFG_FABTECH,
    memtech   => CFG_MEMTECH,
    padtech   => CFG_PADTECH,
    clktech   => CFG_CLKTECH,
    disas     => CFG_DISAS,	-- Enable disassembly to console
    dbguart   => CFG_DUART,	-- Print UART on console
    pclow     => CFG_PCLOW
  )
  port map(
    resetn	=> resetn,       --: key[0]
    clock_50	=> clock_50,
    err_o       => s_err,
    errorn	=> open,-- ledg_o(0),    --: ledr[0], error from LEON3 DSU
    uart_txd  	=> open,		  --: DSU, uart tx data
    uart_rxd  	=> '1',		  --: DSU,uart rx data
    dsubre  	=> '0',       --: key[1], used to put processor in debug mode., deactivated
    dsuact  	=> open,      --: ledr[1]
    i2c_scl	=> i2c_scl_io,
    i2c_sda	=> i2c_sda_io,
    gpio_0_final      => s_gpio_0,      --: inout std_logic_vector(CFG_GRGPIO_WIDTH-1 downto 0); 	-- I/O port 0
    gpio_1      => s_gpio_1,      --: inout std_logic_vector(CFG_GRGPIO2_WIDTH-1 downto 0); 	-- I/O port 1  
    sw      	=> "000",          --: in std_logic_vector(0 to 2) := "000" -- switch for jumper configuratioin, connects debug uart when sw(0) = '0'
		ledg => ledg_o,
		dipsw_n => user_dipsw_i,
    fault_injection_buttons_async => user_pb_i
    );
	
	-- Stick LED cathodes to GND --
	hsmc_ledg_o_n <= (others => '0');
	hsmc_ledr_o_n <= (others => '0');
	
	-- add design architecture here --

end rtl;
