/**
* i2c.c
*
* Created on: Oct. 24, 2013
* Adapted for Leon3 on: Nov. 19, 2013
*     Author: Christian, Matthias
* Adapted for new driver architecture
*/
#include "inc_usecase_rtc/includes.h"


void init_i2c(I2C_SLV_t *slv, uint16_t speed) {

                                     /*  Disable core          */
	slv->ctr = 0;
                              /*  Set prescaler         */
	slv->prer = speed;
                      /*  Enable core           */
	slv->ctr = I2C_CTR_CORE_ENABLE;
}



int8_t i2c_addr(I2C_SLV_t *slv, uint8_t addr, uint8_t rw) {


	int8_t retval = 0;

         /*  Write address                                    */
	slv->rtxr = (( (addr&0x7F) << 1 ) | rw);
 /*  Generate START condition, set W flag             */
	slv->csr 	= I2C_CMD_REPEATED_START | I2C_CMD_WRITE;
                 /*  Poll for transfer in progress flag to be cleared */
	while(slv->csr & I2C_STATUS_TIP);

     /*  ACK flag must be cleared                         */
	if(slv->csr & I2C_STATUS_RXACK) retval =  -1;

		return retval;
	}


int8_t i2c_write_byte(I2C_SLV_t *slv, uint8_t byte, uint8_t ctrl) {

	int8_t retval = 0;

                             /*  Put data in out buffer                           */
	slv->rtxr = byte;

                   /*  Set flags and write mode                         */
	slv->csr = ctrl | I2C_CMD_WRITE;


               /*  Poll for transfer in progress flag to be cleared */

	while(slv->csr & I2C_STATUS_TIP);

        /*  ACK flag must be cleared                         */
	if(slv->csr & I2C_STATUS_RXACK) retval = -1;

	return retval;

}


int8_t i2c_read_byte(I2C_SLV_t *slv, uint8_t ctrl) {
	
                   /*  Set flags and read mode                          */
	slv->csr = ctrl | I2C_CMD_READ;

               /*  Poll for transfer in progress flag to be cleared */
	while(slv->csr & I2C_STATUS_TIP);


                                  /*  Return the value in the in buffer                */
	return slv->rtxr;
}

int8_t stall_i2c(I2C_SLV_t *slv) {

	int8_t retval = 0;
	i2c_addr(slv, RTC_I2C_ADDRESS, I2C_CMD_WRITE);                          /* START, address RTC, start write access            */
	i2c_write_byte(slv,RTC_HOURS_REG,0);                                    /* Set write pointer to seconds reg (reg2)           */
	i2c_write_byte(slv, 0x00,  I2C_CMD_STOP);                               /* Set 24 hours mode, stop transmission              */

	slv->ctr = 0;
	return retval;
}
