/**
 * i2c.h
 *
 * Created on: Oct. 24, 2013
 * Adapted for leon3 on: Nov. 19, 2013
 *     Author: Christian, Matthias
 * Adapted fro new driver architecture
 */

#include <stdint.h>

#ifndef _I2C_H_
#define _I2C_H_

/*****************************************************************************
 * S Y M B O L I C   C O N S T A N T S                                       *
 *****************************************************************************/

/* Register address definitions */

/*TODO: fill constants for LCD I2C with real address values, now point to RTC */

#define RTC_BASE_ADDR ((uint32_t *) (0x80000C00))
#define LCD_BASE_ADDR ((uint32_t *) (0x80000C00))


#define RTC_I2C_PRER   ((volatile uint32_t *) 0x80000C00)
#define RTC_I2C_CTR    ((volatile uint32_t *) 0x80000C04)
#define RTC_I2C_RTXR   ((volatile uint32_t *) 0x80000C08)
#define RTC_I2C_CSR    ((volatile uint32_t *) 0x80000C0C)

#define LCD_I2C_PRER   ((volatile uint32_t *) 0x80000C00)
#define LCD_I2C_CTR    ((volatile uint32_t *) 0x80000C04)
#define LCD_I2C_RTXR   ((volatile uint32_t *) 0x80000C08)
#define LCD_I2C_CSR    ((volatile uint32_t *) 0x80000C0C)

typedef struct {
	uint32_t prer;
	uint32_t ctr;
	uint32_t rtxr;
	uint32_t csr;
}I2C_SLV_t;


/* I2C Controller Index */

#define RTC_I2C                  0x00
#define LCD_I2C                  0x00

/* I2C Addr RW Bit */

#define I2C_ADDR_WRITE           0
#define I2C_ADDR_READ            1

/* I2C CMD Register Bits */

#define I2C_CMD_NACK            (1 << 3)
#define I2C_CMD_WRITE           (1 << 4)
#define I2C_CMD_READ            (1 << 5)
#define I2C_CMD_STOP            (1 << 6)
#define I2C_CMD_REPEATED_START  (1 << 7)

/* I2C STATUS Register Bits */

#define I2C_STATUS_TIP           (1 << 1)
#define I2C_STATUS_RXACK         (1 << 7)

/* I2C Control Register Bits */

#define I2C_CTR_CORE_ENABLE      (1 << 7)
#define I2C_CTR_INT_ENABLE       (1 << 6)

/* I2C Speed values */

#define I2C_SPEED_50kHz  221 /* (AMBACLK 50 MHz)  225 kHz internal clock enable => 5*45  kHz I2C (was 200, needed to lower scl for LCD stability) */
#define I2C_SPEED_100kHz 99  /* (AMBACLK 50 MHz)  500 kHz internal clock enable => 5*100 kHz I2C */
#define I2C_SPEED_500kHz 19  /* (AMBACLK 50 MHz) 2000 kHz internal clock enable => 5*400 kHz I2C */

/*****************************************************************************
 * T Y P E   D E F I N I T I O N S                                           *
 *****************************************************************************/

struct i2cc_regs {
	volatile uint32_t *PRER;
	volatile uint32_t *CTR;
	volatile uint32_t *RTXR;
	volatile uint32_t *CSR;
};

/*****************************************************************************
 * F U N C T I O N   P R O T O T Y P E S                                     *
 *****************************************************************************/

/*
 * void init_i2c(uint8_t i2cc, uint16_t speed);
 *
 * int8_t i2c_addr(uint8_t i2cc, uint8_t addr, uint8_t rw);
 * int8_t i2c_write_byte(uint8_t i2cc, uint8_t byte, uint8_t ctrl);
 * int8_t i2c_read_byte(uint8_t i2cc, uint8_t ctrl);
 *
 * int8_t stall_i2c(uint8_t i2cc);
 *
 */
void init_i2c(I2C_SLV_t *slv, uint16_t speed);

int8_t i2c_addr(I2C_SLV_t *slv, uint8_t addr, uint8_t rw);
int8_t i2c_write_byte(I2C_SLV_t *slv, uint8_t byte, uint8_t ctrl);
int8_t i2c_read_byte(I2C_SLV_t *slv, uint8_t ctrl);

int8_t stall_i2c(I2C_SLV_t *slv);


#endif
