#include <leonio.h>
#include <irq.h>


/*WORKAROUND, SEE BUG 1*/
volatile uint32_t isr_id = 0;

static uint32_t  this_cpu = 0;
static volatile uint32_t cpus_operational = 0;

isr_i irq_f[15] = {0};

/*targetting mask registers only -> activates interrupt*/
IRQMP_BASE_t *irqmp_base = (IRQMP_BASE_t *)IRQMP0_BASE_ADDR;
IRQMP_MASK_t *irqmp_mask = (IRQMP_MASK_t *)IRQMP0_MASK_ADDR;


uint32_t get_cpuid(void) {
	/*get current processors id*/

	asm volatile("set this_cpu, %%l7\n" :/*regs out*/ : /*regs in*/:/*globber*/"%l7");
	asm volatile ("st %%g4,[%%l7]\n" : : :"%g4","%l7");
	return this_cpu;

}

void _irq_handle(uint32_t id) {
	/*BUG 1: Although _irq_entry obeys the function call conventions of sparcv8
	 * (register %o0 of caller is register %i0 of callee)
	 * the given parameter (id) is not taken correctly by irq_handle()
	 * (param is not stored and obtained from stack in a correct way. should be: 0x8, is 0xa800...)
	 * WORKAROUND: store content of %i0 on isr_id; store back to local variable d. call correct isr
	 */
	(void) id;/*to surpress warnings*/
	register uint32_t d = 0;
	asm volatile("set isr_id, %%l7\n" :/*regs out*/ : /*regs in*/:/*globber*/"%l7");
	asm volatile ("st %%i0,[%%l7]\n" : : :"%l7","%i0");
	d = isr_id;
	/*END WORKAROUND*/
	irq_f[d](d);
}

void init_irqmp(void) {

	/*reset interrupt controller*/
	irqmp_base->clear = 0xffff;
	irqmp_base->level = 0;

}

/*activates the given interrupt (id) on cpu cpuid*/
void activate_interrupt(uint32_t cpuid, uint32_t id) {
	irqmp_mask->mask[cpuid] |= (1<<id);
}

/*registers the given isr at interrupt controller channel id*/
void register_isr(uint32_t id,void (*isr)(uint32_t)) {

	irq_f[id] = isr;
	irqmp_base->level &= ~(1<<id);

}

/*start 2nd cpu, resets from 0x0*/
void start_cpu(uint32_t cpu) {
	/*supports up to 4 cpus*/
	irqmp_base->mpstat |= (1<<(cpu & 0xf));
	/*wait for new cpu until it passed its initialization phase*/
	while(!(cpus_operational & (1<<cpu)));
}

/*registers this cpu as ready for duty*/
void register_cpu(uint32_t id) {
	/*support up to 4 cpus*/
	cpus_operational |= (1<<(id & 0x0f));
}