/**
 * lcd.c
 *
 * Created on: Oct. 28, 2013
 * Adapted for Leon3: Nov 19, 2013
 *     Author: Christian, Matthias
 *  Adapted for new driver architecture
 */

#include "inc_usecase_rtc/includes.h"

void delay_100us(uint32_t m) {
	volatile uint32_t i,j;
	for(j = 0; j < m; j++)
		for(i = 0; i < 145/*for leon3 with 50 MHz*/; i++);
		return;
}

void init_display(I2C_SLV_t *i2c_lcd_slv) {

	i2c_addr(i2c_lcd_slv,DISPLAY_I2C_ADDR,I2C_ADDR_WRITE);                          /* START, address LCD, start write access */
	i2c_write_byte(i2c_lcd_slv,DISPLAY_CMD,0);                                      /* Command byte                           */
	i2c_write_byte(i2c_lcd_slv,DISPLAY_CMD_DISPLAY_ON,I2C_CMD_STOP);                /* Turn display on, STOP                  */

	delay_100us(1);                                                             /* 100 us wait required (see DS)          */

	i2c_addr(i2c_lcd_slv,DISPLAY_I2C_ADDR,I2C_ADDR_WRITE);                          /* START, address RTC, start write access */
	i2c_write_byte(i2c_lcd_slv,DISPLAY_CMD,0);                                      /* Command byte                           */
	i2c_write_byte(i2c_lcd_slv,0x51,I2C_CMD_STOP);                                  /* Clear screen, STOP                     */

	delay_100us(15);                                                            /*  1.5 ms wait required (see DS)         */
}

void display_putc(I2C_SLV_t *i2c_lcd_slv, char c) {

	i2c_addr(i2c_lcd_slv,DISPLAY_I2C_ADDR,I2C_ADDR_WRITE);                          /* START, address RTC, start write access */
	i2c_write_byte(i2c_lcd_slv,c,I2C_CMD_STOP);                                     /* Write character, STOP                  */

}

void display_puts(I2C_SLV_t *i2c_lcd_slv, char const * string, uint8_t pos) {

	i2c_addr(i2c_lcd_slv,DISPLAY_I2C_ADDR,I2C_ADDR_WRITE);                          /* START, address RTC, start write access */
	i2c_write_byte(i2c_lcd_slv,DISPLAY_CMD,0);                                      /* Command byte                           */
	i2c_write_byte(i2c_lcd_slv,DISPLAY_CMD_SET_CURSOR,0);                           /* Set cursor command                     */
	i2c_write_byte(i2c_lcd_slv,pos,I2C_CMD_STOP);                                   /* Set cursor to desired position, STOP   */

	delay_100us(1);                                                             /*  100 us wait required (see DS)         */

	if(*string != '\0')
		i2c_addr(i2c_lcd_slv,DISPLAY_I2C_ADDR,I2C_ADDR_WRITE);                      /*  START if first char not 0             */

		while(*string != '\0') {
			if(*(string+1) == '\0')                                                 /*  If next char is 0, request STOP       */
				i2c_write_byte(i2c_lcd_slv,*string++,I2C_CMD_STOP);                     /*  Write last char                       */
				else
					i2c_write_byte(i2c_lcd_slv,*string++,0);                                /*  Write char                            */
		}

}
