/**
 * rtc.c
 *
 * Created on: Oct. 24, 2013
 *     Author: Christian, Matthias
 *  Adapted for new driver architecture
 */
#include "inc_usecase_rtc/includes.h"


char init_rtc(I2C_SLV_t *i2c_rtc_slv,struct time * set_time) {

	char retval = 0;

	/*
	 *	IE   = 0;
	 */
	//was RTC_I2C
	i2c_addr(i2c_rtc_slv, RTC_I2C_ADDRESS, I2C_ADDR_WRITE);                         /* START, address RTC, start write access         */
	i2c_write_byte(i2c_rtc_slv,RTC_HOURS_REG,0);                                    /* Set write pointer to hours reg (reg2)          */
	i2c_write_byte(i2c_rtc_slv, 0x00,  I2C_CMD_STOP);                               /* Set 24 hours mode, stop transmission           */

	i2c_addr(i2c_rtc_slv, RTC_I2C_ADDRESS, I2C_CMD_WRITE);                          /* START, address RTC, start write access         */
	i2c_write_byte(i2c_rtc_slv, RTC_SECONDS_REG, 0);                                /* Set write pointer to seconds reg (reg0)        */
	i2c_write_byte(i2c_rtc_slv, set_time->sec,  0);                                 /* Write seconds                             (00) */
	i2c_write_byte(i2c_rtc_slv, set_time->min,  0);                                 /* Write minutes                             (01) */
	i2c_write_byte(i2c_rtc_slv, set_time->hrs,  0);                                 /* Write hours                               (02) */
	i2c_write_byte(i2c_rtc_slv, set_time->dow,  0);                                 /* Write day of week                         (03) */
	i2c_write_byte(i2c_rtc_slv, set_time->dom,  0);                                 /* Write day of month (century is always 0)  (04) */
	i2c_write_byte(i2c_rtc_slv, set_time->month,0);                                 /* Write month                               (05) */
	i2c_write_byte(i2c_rtc_slv, set_time->year, 0);                                 /* Write year                                (06) */
	i2c_write_byte(i2c_rtc_slv, 0x00,           0);                                 /* Set ALARM1 sec register                   (07) */
	i2c_write_byte(i2c_rtc_slv, 0x00,           0);                                 /* Set ALARM1 min register                   (08) */
	i2c_write_byte(i2c_rtc_slv, 0x00,           0);                                 /* Set ALARM1 hrs register                   (09) */
	i2c_write_byte(i2c_rtc_slv, 0x00,           0);                                 /* Set ALARM1 date register                  (0A) */
	i2c_write_byte(i2c_rtc_slv, 0x00,           0);                                 /* Set ALARM2 min register                   (0B) */
	i2c_write_byte(i2c_rtc_slv, 0x00,           0);                                 /* Set ALARM2 hrs register                   (0C) */
	i2c_write_byte(i2c_rtc_slv, 0x00,           0);                                 /* Set ALARM2 date register                  (0D) */
	i2c_write_byte(i2c_rtc_slv, 0x00,           0);                                 /* Set CONTROL register                      (0E) */
	i2c_write_byte(i2c_rtc_slv, 0x00,           0);                                 /* Ack STATUS flags                          (0F) */
	i2c_write_byte(i2c_rtc_slv, 0x00,           I2C_CMD_STOP);                      /* Disable Trickle Charge                    (10) */

	/*
	 *	IE   = (1 << 7) | (1 << 0);
	 */

	return retval;

}

char read_rtc(I2C_SLV_t *i2c_rtc_slv,struct time * read_time) {

	char retval = 0;

	i2c_addr(i2c_rtc_slv, RTC_I2C_ADDRESS, I2C_ADDR_WRITE);                         /* START, address RTC, start write access     */
	i2c_write_byte(i2c_rtc_slv, RTC_SECONDS_REG, 0);                                /* Set write pointer to seconds reg (reg0)    */
	i2c_addr(i2c_rtc_slv, RTC_I2C_ADDRESS, I2C_ADDR_READ);                          /* START, address RTC, start read access      */

	read_time->sec  = i2c_read_byte(i2c_rtc_slv,0);                                 /* Read seconds                               */
	read_time->min  = i2c_read_byte(i2c_rtc_slv,0);                                 /* Read minutes                               */
	read_time->hrs  = i2c_read_byte(i2c_rtc_slv,0);                                 /* Read hours                                 */
	read_time->dow  = i2c_read_byte(i2c_rtc_slv,0);                                 /* Read day of week                           */
	read_time->dom  = i2c_read_byte(i2c_rtc_slv,0) & 0x7F;                          /* Read day of month (Century bit is ignored) */
	read_time->year = i2c_read_byte(i2c_rtc_slv,I2C_CMD_STOP | I2C_CMD_NACK);       /* Read year, stop transmission               */

	return retval;
}
