------------------------------------------------------------------------------
--  LEON3 Demonstration design test bench
--  Copyright (C) 2004 Jiri Gaisler, Gaisler Research
------------------------------------------------------------------------------
------------------------------------------------------------------------------
--  This file is a part of the GRLIB VHDL IP LIBRARY
--  Copyright (C) 2003 - 2008, Gaisler Research
--  Copyright (C) 2008 - 2013, Aeroflex Gaisler
--
--  This program is free software; you can redistribute it and/or modify
--  it under the terms of the GNU General Public License as published by
--  the Free Software Foundation; either version 2 of the License, or
--  (at your option) any later version.
--
--  This program is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--  GNU General Public License for more details.
--
--  You should have received a copy of the GNU General Public License
--  along with this program; if not, write to the Free Software
--  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

--Adapted for the use on the LEON3 Usecase at the JRZ VECS - FH Technikum Wien (c) 2013/2014

library ieee;
use ieee.std_logic_1164.all;
library gaisler;
use gaisler.libdcom.all;
use gaisler.sim.all;
use work.debug.all;
library techmap;
use techmap.gencomp.all;
library micron;
use micron.components.all;
library grlib;
use grlib.stdlib.all;

use work.config.all;	-- configuration
use work.fault_injection_p.all;


entity testbench is
  generic (
    fabtech   : integer := CFG_FABTECH;
    memtech   : integer := CFG_MEMTECH;
    padtech   : integer := CFG_PADTECH;
    clktech   : integer := CFG_CLKTECH;
    disas     : integer := CFG_DISAS;	-- Enable disassembly to console
    dbguart   : integer := CFG_DUART;	-- Print UART on console
    pclow     : integer := CFG_PCLOW;

    clkperiod : integer := 20		-- system clock period
  );
end; 

architecture behav of testbench is

signal clk : std_logic := '0';
signal Rst    : std_logic := '0';			-- Reset
constant ct : integer := clkperiod/2;


signal dsuen, dsutx, dsurx, dsubre, dsuact : std_logic;
signal dsurst   : std_logic;
signal error    : std_logic;
signal err_o 	:std_logic_vector(3 downto 0);
signal gpio_0	: std_logic_vector(CFG_GRGPIO_WIDTH-1 downto 0);
signal gpio_1	: std_logic_vector(CFG_GRGPIO2_WIDTH-1 downto 0);
    
signal i2c_scl  : std_logic;
signal i2c_sda  : std_logic;

signal sw : std_logic_vector(0 to 2);      


constant lresp : boolean := false;



--i2c slave model


component i2c_rtc_slave_model is
	port(
		scl : in std_logic;
		sda : inout std_logic
	);
end component;
  
--end i2c slave model


signal fault_injection_buttons : std_logic_vector(1 downto 0) := (others => not(C_BUTTON_PRESSED));
constant FI_SWITCH_ON : std_logic := C_BUTTON_PRESSED;
signal dipsw_n :std_logic_vector(3 downto 0) := (others => not (FI_SWITCH_ON));
signal ledg :std_logic_vector(3 downto 0);

constant SIMULATION_ON :integer := 1;

--presses refelct leds on board
procedure SELECT_FI 
 (constant address : in integer;
	signal fi_button_0 : out std_logic ) is
 variable i :integer := 0;
begin
	while (i <  address) loop 
		fi_button_0 <= C_BUTTON_PRESSED;
		wait for 100 ns;
		fi_button_0 <= not C_BUTTON_PRESSED;
		wait for 100 ns;
		i := i + 1;
	end loop;
end procedure SELECT_FI;

procedure INJECT_FAULT_IN_US 
	(constant length :in integer;
	 signal fi_button_1 :out std_logic) is
	 variable i :integer := 0;
begin
	fi_button_1 <= C_BUTTON_PRESSED;
	while i < length loop
		wait for 1 us;
		i := i + 1;
	end loop;
	fi_button_1 <= not C_BUTTON_PRESSED;
end procedure INJECT_FAULT_IN_US;

begin

  clk <= not clk after ct * 1 ns; --50 MHz clk 
  rst <= dsurst; --reset
  dsuen <= '1';
  dsubre <= '1'; -- inverted on the board
  sw(0) <= '1';
  gpio_0(CFG_GRGPIO_WIDTH-1 downto 0) <= (others => 'H');
  gpio_1(CFG_GRGPIO2_WIDTH-1 downto 0) <= (others => 'H');

  d3 : entity work.leon3mp
        generic map ( fabtech, memtech, padtech, clktech, disas, dbguart, pclow, SIMULATION_ON)
        port map (rst, clk,err_o, error, dsutx, dsurx, dsubre, dsuact,
	i2c_scl,i2c_sda, gpio_0, gpio_1, fault_injection_buttons,ledg,dipsw_n,sw);
	
 i2c_rtc_slave: entity work.i2c_rtc_slave_model
    port map(scl => i2c_scl,
	     sda => i2c_sda
	  );
	
    i2c_scl <= 'H';
    i2c_sda <= 'H';
	


  error <= 'H';			  -- ERROR pull-up

	fi_proc : process
	begin

	wait for 1 ms;

	--test TRIGGER_SUCCESSIVE_INTERRUPT
--	SELECT_FI(1,fault_injection_buttons(0));
--	INJECT_FAULT_IN_US(200,fault_injection_buttons(1));
--	wait for 1 ms;
	
	--test TRIGGER_RULE5_ADDR_STABLE_DURING_TRANSITION - causes system reset
--	SELECT_FI(4+15,fault_injection_buttons(0));
--	INJECT_FAULT_IN_US(200,fault_injection_buttons(1));
--	wait for 2 ms;

	wait for 1 ms;--added for this ctsn test only!	
	--test TRIGGER_CTSN
	SELECT_FI(2,fault_injection_buttons(0)); -- button decoder is also reset
	INJECT_FAULT_IN_US(200,fault_injection_buttons(1));
	wait for 2 ms;

	--test TRIGGER_RULE22_HTRANS_CHANGE_WHEN_HREADY_LOW - causes system reset
--	SELECT_FI(3+(15-1),fault_injection_buttons(0));
--	INJECT_FAULT_IN_US(200,fault_injection_buttons(1));
	
	wait;
	end process;
  
  
  
   iuerr : process
   begin
     wait for 2500 ns;
     if to_x01(error) = '1' then wait on error; end if;
     assert (to_x01(error) = '1') 
       report "*** IU in error mode, simulation halted ***"
         severity failure ;
   end process;


  dsucom : process
    procedure dsucfg(signal dsurx : in std_logic; signal dsutx : out std_logic) is
    variable w32 : std_logic_vector(31 downto 0);
    variable c8  : std_logic_vector(7 downto 0);
    constant txp : time := 160 * 1 ns;
    begin
    dsutx <= '1';
    dsurst <= '0'; --reset low
    wait for 500 ns;
    dsurst <= '1'; --reset high
    wait; --evig w8
    wait for 5000 ns;
    txc(dsutx, 16#55#, txp);

--    txc(dsutx, 16#c0#, txp); --control byte
--    txa(dsutx, 16#90#, 16#00#, 16#00#, 16#00#, txp); --adress
--    txa(dsutx, 16#00#, 16#00#, 16#02#, 16#ae#, txp); --write data
--    txc(dsutx, 16#c0#, txp);
--    txa(dsutx, 16#91#, 16#00#, 16#00#, 16#00#, txp);
--    txa(dsutx, 16#00#, 16#00#, 16#06#, 16#ae#, txp);
--    txc(dsutx, 16#c0#, txp);
--    txa(dsutx, 16#90#, 16#00#, 16#00#, 16#24#, txp);
--    txa(dsutx, 16#00#, 16#00#, 16#06#, 16#03#, txp);
--    txc(dsutx, 16#c0#, txp);
--    txa(dsutx, 16#90#, 16#00#, 16#00#, 16#20#, txp);
--    txa(dsutx, 16#00#, 16#00#, 16#06#, 16#fc#, txp);

    txc(dsutx, 16#c0#, txp);
    txa(dsutx, 16#90#, 16#00#, 16#00#, 16#00#, txp);
    txa(dsutx, 16#00#, 16#00#, 16#00#, 16#2f#, txp);
    txc(dsutx, 16#c0#, txp);
    txa(dsutx, 16#91#, 16#00#, 16#00#, 16#00#, txp);
    txa(dsutx, 16#00#, 16#00#, 16#00#, 16#6f#, txp);
    txc(dsutx, 16#c0#, txp);
    txa(dsutx, 16#90#, 16#11#, 16#00#, 16#00#, txp);
    txa(dsutx, 16#00#, 16#00#, 16#00#, 16#00#, txp);
    txc(dsutx, 16#c0#, txp);
    txa(dsutx, 16#90#, 16#40#, 16#00#, 16#04#, txp);
    txa(dsutx, 16#00#, 16#02#, 16#20#, 16#01#, txp);
    txc(dsutx, 16#c0#, txp);
    txa(dsutx, 16#90#, 16#00#, 16#00#, 16#20#, txp);
    txa(dsutx, 16#00#, 16#00#, 16#00#, 16#02#, txp);

    txc(dsutx, 16#c0#, txp);
    txa(dsutx, 16#90#, 16#00#, 16#00#, 16#20#, txp);
    txa(dsutx, 16#00#, 16#00#, 16#00#, 16#0f#, txp);

    txc(dsutx, 16#c0#, txp);
    txa(dsutx, 16#40#, 16#00#, 16#43#, 16#10#, txp);
    txa(dsutx, 16#00#, 16#00#, 16#00#, 16#0f#, txp);

    txc(dsutx, 16#c0#, txp);
    txa(dsutx, 16#91#, 16#40#, 16#00#, 16#24#, txp);
    txa(dsutx, 16#00#, 16#00#, 16#00#, 16#24#, txp);
    txc(dsutx, 16#c0#, txp);
    txa(dsutx, 16#91#, 16#70#, 16#00#, 16#00#, txp);
    txa(dsutx, 16#00#, 16#00#, 16#00#, 16#03#, txp);





    txc(dsutx, 16#c0#, txp);
    txa(dsutx, 16#90#, 16#00#, 16#00#, 16#20#, txp);
    txa(dsutx, 16#00#, 16#00#, 16#ff#, 16#ff#, txp);

    txc(dsutx, 16#c0#, txp);
    txa(dsutx, 16#90#, 16#40#, 16#00#, 16#48#, txp);
    txa(dsutx, 16#00#, 16#00#, 16#00#, 16#12#, txp);

    txc(dsutx, 16#c0#, txp);
    txa(dsutx, 16#90#, 16#40#, 16#00#, 16#60#, txp);
    txa(dsutx, 16#00#, 16#00#, 16#12#, 16#10#, txp);

    txc(dsutx, 16#80#, txp);
    txa(dsutx, 16#90#, 16#00#, 16#00#, 16#00#, txp);
    rxi(dsurx, w32, txp, lresp);

    txc(dsutx, 16#a0#, txp);
    txa(dsutx, 16#40#, 16#00#, 16#00#, 16#00#, txp);
    rxi(dsurx, w32, txp, lresp);

    end;

  begin

    dsucfg(dsutx, dsurx);

    wait;
 end process;

  p_sim : process
  begin 
    wait for 2 ms;
    assert false report "End of Simulation!" severity failure;
  end process;

end ;

